unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    CardList: TListBox;
    Splitter1: TSplitter;
    Panel2: TPanel;
    IOList: TListBox;
    IOEdit: TEdit;
    Splitter2: TSplitter;
    Panel1: TPanel;
    ResultMemo: TMemo;
    IRQCardEdit: TEdit;
    IRQWaitButton: TButton;
    IRQWaitDataButton: TButton;
    WaitLabel: TLabel;
    procedure IOListDblClick(Sender: TObject);
    procedure IOEditKeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure IRQWaitButtonClick(Sender: TObject);
    procedure IRQWaitDataButtonClick(Sender: TObject);
  protected
    procedure DoIO(IOCmd: String);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  Math, DIO247, WaitThreads;

procedure TMainForm.IOListDblClick(Sender: TObject);
var
  Buf: String;
  I: Integer;

begin
  I := IOList.ItemIndex;
  if I < 0 then Exit;
  Buf := IOList.Items[I];
  IOList.ItemIndex := -1;
  DoIO(Buf);
end;

procedure TMainForm.IOEditKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then begin
    Key := #0;
    DoIO(IOEdit.Text);
  end;
end;

procedure TMainForm.DoIO(IOCmd: String);
var
  I, J, Rel, Size: Integer;
  AddrStr, DataStr, ResultStr: String;
  Addr, Data: LongWord;

begin
  I := Pos('+', IOCmd);
  if I = 0 then
    Rel := -1
  else
  begin
    Rel := StrToInt('$' + Copy(IOCmd, 1, I - 1));
    IOCmd := Copy(IOCmd, I + 1, MAXINT);
  end;

  I := Pos('<', IOCmd);
  J := Pos('>', IOCmd);
  if J = 0 then
    //
  else if (I = 0) or (J < I) then
    I := J
  ;
  if I = 0 then begin
    ResultStr := 'Error in "' + IOCmd + '".';
  end
  else begin
    AddrStr := Copy(IOCmd, 1, I - 1);
    DataStr := Copy(IOCmd, I + 1, MAXINT);
    Addr := StrToInt('$' + AddrStr);
    Data := StrToInt('$' + DataStr);
    Size := Ceil(Length(DataStr) / 2);
    if Size = 0 then Size := 1 else if Size = 3 then Size := 4;
    if Rel < 0 then begin
      if IOCmd[I] = '<' then begin
        case Size of
          1: begin
            Data := Data and $FF;
            OutPortB(Addr, Data);
          end;
          2: begin
            Data := Data and $FFFF;
            OutPort(Addr, Data);
          end;
          else begin
            OutPortL(Addr, Data);
          end;
        end;
        ResultStr := IntToHex(Addr, 1) + '<' + IntToHex(Data, 2 * Size);
      end
      else begin
        case Size of
          1: Data := InPortB(Addr);
          2: Data := InPort(Addr);
          else Data := InPortL(Addr);
        end;
        ResultStr := IntToHex(Addr, 1) + '>' + IntToHex(Data, 2 * Size);
      end;
    end
    else begin
      if IOCmd[I] = '<' then begin
        case Size of
          1: begin
            Data := Data and $FF;
            RelOutPortB(Rel, Addr, Data);
          end;
          2: begin
            Data := Data and $FFFF;
            RelOutPort(Rel, Addr, Data);
          end;
          else begin
            RelOutPortL(Rel, Addr, Data);
          end;
        end;
        ResultStr := IntToHex(Rel, 1) + '+' + IntToHex(Addr, 1) + '<' + IntToHex(Data, 2 * Size);
      end
      else begin
        case Size of
          1:   Data := RelInPortB(Rel, Addr);
          2:   Data := RelInPort(Rel, Addr);
          else Data := RelInPortL(Rel, Addr);
        end;
        ResultStr := IntToHex(Rel, 1) + '+' + IntToHex(Addr, 1) + '>' + IntToHex(Data, 2 * Size);
      end;
    end;
  end;
  I := IOList.Items.Add(ResultStr);
  IOList.ItemIndex := I;
  IOList.ItemIndex := -1;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  I, L: Integer;
  Status, ID, Base: LongWord;

begin
  IOEdit.Align := alBottom;
  L := GetNumCards;
  ResultMemo.Lines.Add(IntToStr(L) + ' cards detected');
  for I := 0 to L - 1 do begin
    Status := QueryCardID(I, @ID);
    if Status <> ERROR_SUCCESS then begin
      CardList.Items.Add(IntToHex(I, 1) + ': error ' + IntToStr(Status) + ' in QueryCardID.');
      Continue;
    end;
    Status := QueryCardBase(I, @Base);
    if Status <> ERROR_SUCCESS then begin
      CardList.Items.Add(IntToHex(I, 1) + ': error ' + IntToStr(Status) + ' in QueryCardBase.');
      Continue;
    end;
    CardList.Items.Add(IntToHex(I, 1) + ': ' + IntToHex(ID, 1) + ' @ ' + IntToHex(Base, 1));
  end;
end;

procedure TMainForm.IRQWaitButtonClick(Sender: TObject);
var
  I: Integer;

begin
  I := StrToInt('$' + IRQCardEdit.Text);
  TWaitThread.Launch(I);
end;

procedure TMainForm.IRQWaitDataButtonClick(Sender: TObject);
var
  I: Integer;

begin
  I := StrToInt('$' + IRQCardEdit.Text);
  TWaitDataThread.Launch(I);
end;

end.
