unit WaitThreads;

interface

uses
  SysUtils, Classes;

type
  TWaitThread = class(TThread)
  protected
    CardNum: Integer;
    WaitResult: LongWord;

    procedure Execute; override;
    procedure ReportData; virtual;
  public
    class function Launch(NewCardNum: Integer): TWaitThread; virtual;
  end;

  TWaitDataThread = class(TWaitThread)
  protected
    DIOData: array of Byte;

    procedure Execute; override;
    procedure ReportData; override;
  end;





implementation

uses
  Windows, DIO247, MainUnit;

const
  PPIBytes = 3;

var
  WaitList: array of TWaitThread;

procedure ListWaits;
var
  I, L: Integer;
  Buf: String;

begin
  L := Length(WaitList);
  if L = 0 then
    Buf := 'No Waiting'
  else begin
    Buf := 'Waiting:'#13#10;
    for I := 0 to L - 1 do
      Buf := Buf + WaitList[I].ClassName + #13#10
    ;
  end;
  MainForm.WaitLabel.Caption := Buf;
end;

procedure ClearWait(LeavingThread: TWaitThread);
var
  H, I: Integer;

begin
  H := High(WaitList);
  for I := H downto 0 do
    if WaitList[I] = LeavingThread then begin
      if I < H then Move(WaitList[I + 1], WaitList[I], 4 * (H - I));
      Dec(H);
    end
  ;
  SetLength(WaitList, H + 1);

  ListWaits;
end;

{ TWaitThread }

procedure TWaitThread.Execute;
begin
  WaitResult := WaitForIRQ(CardNum);
  Synchronize(ReportData);
end;

class function TWaitThread.Launch(NewCardNum: Integer): TWaitThread;
var
  I: Integer;

begin
  Result := Self.Create(True);
  Result.CardNum := NewCardNum;
  Result.FreeOnTerminate := True;

  I := Length(WaitList);
  SetLength(WaitList, I + 1);
  WaitList[I] := Result;
  ListWaits;

  Result.Resume;
end;

procedure TWaitThread.ReportData;
begin
  MainForm.ResultMemo.Lines.Add('WaitForIRQ got ' + IntToStr(WaitResult));
  ClearWait(Self);
end;

{ TWaitDataThread }

procedure TWaitDataThread.Execute;
begin
  SetLength(DIOData, PPIBytes);
  WaitResult := WaitForIRQData(CardNum, Length(DIOData) * SizeOf(DIOData[0]), @DIOData[0]);
  Synchronize(ReportData);
end;

procedure TWaitDataThread.ReportData;
var
  I: Integer;
  Buf: String;

begin
  MainForm.ResultMemo.Lines.Text := 'WaitForIRQData got ' + IntToStr(WaitResult) + #13#10;
  if WaitResult = ERROR_SUCCESS then begin
    MainForm.ResultMemo.Lines.Add('Lo the data:');
    Buf := '';
    for I := 0 to PPIBytes - 1 do
      Buf := Buf + ' ' + IntToHex(DIOData[I], 2)
    ;
    MainForm.ResultMemo.Lines.Add(Buf);
  end;
  ClearWait(Self);
end;

end.
